from mage_ai.data_cleaner.column_types.constants import NUMBER_TYPES, ColumnType
from mage_ai.shared.multi import run_parallel_multiple_args
from mage_ai.data_cleaner.transformer_actions.constants import (
    CONSTANT_IMPUTATION_DEFAULTS,
    INVALID_VALUE_PLACEHOLDERS,
)
import numpy as np
import pandas as pd
import re

DATETIME_MATCHES_THRESHOLD = 0.5
MAXIMUM_WORD_LENGTH_FOR_CATEGORY_FEATURES = 40
MAX_ROWS_FOR_COLUMN_TYPE_INFERENCE = 100_000
MULTITHREAD_MAX_NUM_ENTRIES = 50000
NUMBER_TYPE_MATCHES_THRESHOLD = 0.8
STRING_TYPE_MATCHES_THRESHOLD = 0.3

REGEX_DATETIME_PATTERN = r'^\d{2,4}-\d{1,2}-\d{1,2}$|^\d{2,4}-\d{1,2}-\d{1,2}[Tt ]{1}\d{1,2}:\d{1,2}[:]{0,1}\d{1,2}[\.]{0,1}\d*|^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z$|^\d{1,4}[-\/]{1}\d{1,2}[-\/]{1}\d{1,2}$|^\d{1,2}[-\/]{1}\d{1,2}[-\/]{1}\d{1,4}$|^\d{1,2}[-\/]\d{1,2}[-\/]\d{2,4}[Tt ]{1}\d{1,2}:\d{1,2}[:]{0,1}\d{1,2}[\.]{0,1}\d*|(Jan(?:uary)?|Feb(?:ruary)?|Mar(?:ch)?|Apr(?:il)?|May|Jun(?:e)?|Jul(?:y)?|Aug(?:ust)?|Sep(?:tember)?|Oct(?:ober)?|Nov(?:ember)?|Dec(?:ember)?)\s+(\d{1,2})[\s,]+(\d{2,4})'
REGEX_DATETIME = re.compile(REGEX_DATETIME_PATTERN)
REGEX_EMAIL_PATTERN = r'^[a-zA-Z0-9_.+#-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$'
REGEX_EMAIL = re.compile(REGEX_EMAIL_PATTERN)
REGEX_INTEGER_PATTERN = r'^\-{0,1}\s*(?:(?:[$€¥₹£]|Rs|CAD){0,1}\s*(?:[0-9]+(?:,[0-9]+)*|[0-9]+){0,1}(?:\.0*)?|(?:[0-9]+(?:,[0-9]+)*|[0-9]+){0,1}(?:\.0*)?\s*(?:[元€$]|CAD){0,1})$'
REGEX_INTEGER = re.compile(REGEX_INTEGER_PATTERN)
REGEX_LIST_PATTERN = r'^[\[(](?:[^\n\r],?)*[\])]$'
REGEX_LIST = re.compile(REGEX_LIST_PATTERN)
REGEX_NUMBER_PATTERN = r'^\-{0,1}\s*(?:(?:[$€¥₹£]|Rs|CAD){0,1}\s*(?:[0-9]+(?:,[0-9]+)*|[0-9]+){0,1}(?:\.[0-9]*){0,1}|(?:[0-9]+(?:,[0-9]+)*|[0-9]+){0,1}(?:\.[0-9]*){0,1}\s*(?:[元€$]|CAD){0,1})\s*\%{0,1}$'
REGEX_NUMBER = re.compile(REGEX_NUMBER_PATTERN)
REGEX_PHONE_NUMBER_PATTERN = (
    r'^\s*(?:\+?(\d{1,3}))?[-. (]*(\d{3})[-. )]*(\d{3})[-. ]*(\d{4})(?: *x(\d+))?\s*$'
)
REGEX_PHONE_NUMBER = re.compile(REGEX_PHONE_NUMBER_PATTERN)
REGEX_ZIP_CODE_PATTERN = r'^\d{3,5}(?:[-\s]\d{4})?$'
REGEX_ZIP_CODE = re.compile(REGEX_ZIP_CODE_PATTERN)

RESERVED_PHONE_NUMBER_WORDS = frozenset(['phone', 'landline'])
RESERVED_ZIP_CODE_WORDS = frozenset(['zip', 'postal', 'zipcode', 'postcode'])


def str_in_set(string, string_set):
    return any(entry in string for entry in string_set)


def find_syntax_errors(series, column_type):
    if len(series) == 0:
        return pd.Series([])

    dtype = series.dtype
    str_series = series
    filter_invalid, filter_null = False, False
    check_syntax_errors = True
    if column_type == ColumnType.EMAIL:
        filter_invalid, filter_null = True, True
        pattern = REGEX_EMAIL
    elif column_type == ColumnType.PHONE_NUMBER:
        str_series = str_series.astype(str)
        filter_invalid, filter_null = True, True
        pattern = REGEX_PHONE_NUMBER
    elif column_type == ColumnType.ZIP_CODE:
        str_series = str_series.astype(str)
        filter_invalid, filter_null = True, True
        pattern = REGEX_ZIP_CODE
    elif (
        column_type in NUMBER_TYPES
        and not np.issubdtype(dtype, np.integer)
        and not np.issubdtype(dtype, np.floating)
    ):
        str_series = str_series.astype(str)
        pattern = REGEX_NUMBER
    elif column_type == ColumnType.DATETIME:
        if type(dtype) is pd.core.dtypes.dtypes.DatetimeTZDtype:
            dtype = dtype.base
        if not (np.issubdtype(dtype, np.datetime64) or dtype is pd.Timestamp):
            str_series = str_series.astype(str)
            pattern = REGEX_DATETIME
        else:
            check_syntax_errors = False
    else:
        check_syntax_errors = False
        pattern = None

    mask = pd.Series([False] * len(series))
    mask.index = series.index
    if check_syntax_errors:
        mask |= ~str_series.str.match(pattern, na=True)
        if filter_invalid:
            mask &= ~str_series.str.match(INVALID_VALUE_PLACEHOLDERS[column_type], na=True)
        if filter_null:
            mask &= ~str_series.str.match(CONSTANT_IMPUTATION_DEFAULTS[column_type], na=True)
    return mask & series.notna()


def infer_number_type(series, column_name, dtype):
    clean_series = series.dropna()
    length = len(clean_series)
    if length == 0:
        mdtype = ColumnType.NUMBER_WITH_DECIMALS
    else:
        is_integer = np.floor(clean_series) == clean_series
        correct_phone_nums = ((clean_series >= 1e9) & (clean_series < 1e12) & is_integer).sum()
        if (
            correct_phone_nums / length >= NUMBER_TYPE_MATCHES_THRESHOLD
            and 'phone' in column_name.lower()
        ):
            mdtype = ColumnType.PHONE_NUMBER
        else:
            if np.issubdtype(dtype, np.integer):
                if (
                    clean_series.min() >= 100
                    and clean_series.max() <= 99999
                    and str_in_set(column_name.lower(), RESERVED_ZIP_CODE_WORDS)
                ):
                    mdtype = ColumnType.ZIP_CODE
                else:
                    mdtype = ColumnType.NUMBER
            elif np.issubdtype(dtype, np.floating):
                if all(is_integer):
                    mdtype = ColumnType.NUMBER
                else:
                    mdtype = ColumnType.NUMBER_WITH_DECIMALS
    return mdtype


def infer_column_type(series, column_name, dtype, kwargs):
    mdtype = None
    if 'datetime64' in str(dtype):
        mdtype = ColumnType.DATETIME
    elif dtype == 'object':
        mdtype = infer_object_type(series, column_name, kwargs)
    elif dtype == 'bool':
        mdtype = ColumnType.TRUE_OR_FALSE
    elif np.issubdtype(dtype, np.floating) or np.issubdtype(dtype, np.integer):
        mdtype = infer_number_type(series, column_name, dtype)
    if mdtype in NUMBER_TYPES and series.nunique(dropna=False) == 2:
        mdtype = ColumnType.TRUE_OR_FALSE

    return mdtype


def infer_object_type(series, column_name, kwargs):
    clean_series = series.apply(lambda x: x.strip(' \'\"') if type(x) is str else x)
    clean_series = clean_series.map(lambda x: x if (not isinstance(x, str) or x != '') else np.nan)
    clean_series = clean_series.dropna()

    exact_dtype = type(clean_series.iloc[0]) if clean_series.count() else None
    if exact_dtype in [list, tuple, set]:
        return ColumnType.LIST

    series_nunique = series.nunique(dropna=False)
    clean_series_nunique = clean_series.nunique()
    if np.issubdtype(exact_dtype, np.bool_):
        if clean_series_nunique <= 2:
            return ColumnType.TRUE_OR_FALSE
        else:
            return ColumnType.CATEGORY
    elif clean_series_nunique <= 2:
        return ColumnType.TRUE_OR_FALSE

    clean_series = clean_series.astype(str)
    length = len(clean_series)
    if all(clean_series.str.match(REGEX_NUMBER)):
        if not all(clean_series.str.match(REGEX_INTEGER)):
            return ColumnType.NUMBER_WITH_DECIMALS
        else:
            lowercase_column_name = column_name.lower()
            correct_phone_nums = clean_series.str.match(REGEX_PHONE_NUMBER).sum()
            correct_zip_codes = clean_series.str.match(REGEX_ZIP_CODE).sum()
            if correct_phone_nums / length >= NUMBER_TYPE_MATCHES_THRESHOLD and str_in_set(
                lowercase_column_name, RESERVED_PHONE_NUMBER_WORDS
            ):
                return ColumnType.PHONE_NUMBER
            elif correct_zip_codes / length >= NUMBER_TYPE_MATCHES_THRESHOLD and str_in_set(
                lowercase_column_name, RESERVED_ZIP_CODE_WORDS
            ):
                return ColumnType.ZIP_CODE
            else:
                clean_series = clean_series.str.replace(r'\.0*', '')
                try:
                    clean_series.astype(int)
                    return ColumnType.NUMBER
                except OverflowError:
                    if clean_series_nunique <= kwargs.get('category_cardinality_threshold', 255):
                        return ColumnType.CATEGORY
                    else:
                        return ColumnType.CATEGORY_HIGH_CARDINALITY

    else:
        matches = clean_series.str.match(REGEX_DATETIME).sum()
        if matches / length >= DATETIME_MATCHES_THRESHOLD:
            return ColumnType.DATETIME
        # TODO: Refactor / Reduce cleaning logic
        correct_emails = clean_series.str.match(REGEX_EMAIL).sum()
        correct_phone_nums = clean_series.str.match(REGEX_PHONE_NUMBER).sum()
        correct_zip_codes = clean_series.str.match(REGEX_ZIP_CODE).sum()
        correct_lists = clean_series.str.match(REGEX_LIST).sum()
        lowercase_column_name = column_name.lower()
        if correct_emails / length >= STRING_TYPE_MATCHES_THRESHOLD:
            return ColumnType.EMAIL
        elif correct_phone_nums / length >= STRING_TYPE_MATCHES_THRESHOLD and str_in_set(
            lowercase_column_name, RESERVED_PHONE_NUMBER_WORDS
        ):
            return ColumnType.PHONE_NUMBER
        elif correct_zip_codes / length >= STRING_TYPE_MATCHES_THRESHOLD and str_in_set(
            lowercase_column_name, RESERVED_ZIP_CODE_WORDS
        ):
            return ColumnType.ZIP_CODE
        elif correct_lists / length >= STRING_TYPE_MATCHES_THRESHOLD:
            return ColumnType.LIST
        elif series_nunique == 2:
            return ColumnType.TRUE_OR_FALSE

        if clean_series_nunique / length >= 0.8:
            return ColumnType.TEXT

        word_count = clean_series.map(lambda x: len(str(x).split(' '))).max()
        if word_count > MAXIMUM_WORD_LENGTH_FOR_CATEGORY_FEATURES:
            return ColumnType.TEXT

        if clean_series_nunique <= kwargs.get('category_cardinality_threshold', 255):
            return ColumnType.CATEGORY
        else:
            return ColumnType.CATEGORY_HIGH_CARDINALITY


def infer_column_types(df, **kwargs):
    column_types = kwargs.get('column_types', {})
    df_columns = df.columns.tolist()
    ctypes = {k: v for k, v in column_types.items() if k in df_columns}
    new_cols = [col for col in df_columns if col not in column_types]
    kwarg_list = [kwargs] * len(new_cols)

    num_entries = len(df)
    if num_entries > MAX_ROWS_FOR_COLUMN_TYPE_INFERENCE:
        df_sample = df.sample(MAX_ROWS_FOR_COLUMN_TYPE_INFERENCE)
    else:
        df_sample = df
    columns = [df_sample[col] for col in new_cols]

    if num_entries > MULTITHREAD_MAX_NUM_ENTRIES:
        types = run_parallel_multiple_args(
            infer_column_type, columns, new_cols, df_sample.dtypes[new_cols], kwarg_list
        )
    else:
        types = map(infer_column_type, columns, new_cols, df_sample.dtypes[new_cols], kwarg_list)
    for col, dtype in zip(new_cols, types):
        ctypes[col] = dtype
    return ctypes



from mage_ai.data_cleaner.transformer_actions.spark.constants import (
    COLUMN_TYPE_MAPPING,
    GROUP_MOD_COLUMN,
    ROW_NUMBER_COLUMN,
    ROW_NUMBER_LIT_COLUMN,
)
from mage_ai.data_cleaner.transformer_actions.utils import clean_column_name
from pyspark.sql import functions as F
from pyspark.sql.functions import PandasUDFType, expr, pandas_udf
from pyspark.sql.types import (
    StringType,
    StructField,
    StructType,
)
from pyspark.sql.window import Window
import re


def __create_new_struct_fields(feature_set, transformer_action):
    new_fields = []

    for output_feature in transformer_action.get('outputs', []):
        if output_feature['uuid'] not in feature_set.schema.fieldNames():
            new_fields.append(StructField(
                output_feature['uuid'],
                COLUMN_TYPE_MAPPING[output_feature['column_type']](),
                True,
            ))
    return new_fields


def transform_add_distance_between(feature_set, transformer_action, sort_options={}):
    from pyspark.sql.functions import col, radians, asin, sin, sqrt, cos
    lat1, lng1, lat2, lng2 = transformer_action['action_arguments']
    output_col = transformer_action['outputs'][0]['uuid']
    return feature_set.withColumn('dlng', radians(col(lng2)) - radians(col(lng1))) \
        .withColumn('dlat', radians(col(lat2)) - radians(col(lat1))) \
        .withColumn(output_col, asin(sqrt(
                                         sin(col('dlat') / 2) ** 2 + cos(radians(col(lat1)))
                                         * cos(radians(col(lat2))) * sin(col('dlng') / 2) ** 2
                                         )
                                     ) * 2 * 3963 * 5280) \
        .drop('dlng', 'dlat')


def transform_agg(feature_set, transformer_action, agg_func, use_string_col_name=False):
    action_options = transformer_action['action_options']
    agg_col = transformer_action['action_arguments'][0]
    if not use_string_col_name:
        agg_col = F.col(agg_col)
    if 'groupby_columns' in action_options and action_options['groupby_columns'] is not None:
        df_agg = (
            feature_set
            .groupBy(action_options['groupby_columns'])
            .agg(*[
                agg_func(agg_col)
                .alias(transformer_action['outputs'][0]['uuid']),
            ])
        )

        return feature_set.join(
                    df_agg,
                    action_options['groupby_columns'],
                    how='left',
                )
    else:
        agg_value = feature_set.agg(*[
            agg_func(agg_col)
            .alias(transformer_action['outputs'][0]['uuid']),
        ])
        return feature_set.crossJoin(agg_value)


def transform_average(feature_set, transformer_action, sort_options={}):
    return transform_agg(feature_set, transformer_action, F.avg)


def transform_clean_column_name(feature_set, transformer_action, sort_options={}):
    columns = transformer_action['action_arguments']
    mapping = {col: clean_column_name(col) for col in columns}
    for old_name, new_name in mapping.items():
        feature_set = feature_set.withColumnRenamed(old_name, new_name)
    return feature_set


def transform_count(feature_set, transformer_action, sort_options={}):
    return transform_agg(feature_set, transformer_action, F.count)


def transform_count_distinct(feature_set, transformer_action, sort_options={}):
    return transform_agg(feature_set, transformer_action, F.countDistinct)


def transform_drop_duplicate(feature_set, transformer_action, sort_options={}):
    if sort_options.get('columns') is not None:
        sort_columns = sort_options.get('columns')
        sort_ascending = sort_options.get('ascending', True)
        if transformer_action['action_options'].get('keep', 'last') == 'last':
            sort_ascending = not sort_ascending
        unique_by_columns = transformer_action['action_arguments']
        rnum_col = 'rnum'

        window = Window.partitionBy(unique_by_columns)
        window = window.orderBy(*[
            F.col(col).asc() if sort_ascending else F.col(col).desc() for col in sort_columns
        ])
        row_number = F.row_number().over(window)

        feature_set = feature_set.withColumn(rnum_col, row_number)
        feature_set = feature_set[feature_set[rnum_col] == 1]

        return feature_set.drop(rnum_col)
    else:
        return feature_set.dropDuplicates(transformer_action['action_arguments'])


def transform_filter(feature_set, transformer_action, sort_options={}):
    action_code = transformer_action['action_code']
    match = re.search(r'^[\w.]+\s{1}', action_code)
    if match:
        column_name = match[0].strip()
    else:
        column_name = None

    if column_name:
        column_type = feature_set.schema[column_name].dataType
        is_string = column_type == StringType
        if f'{column_name} != null' == action_code:
            if is_string:
                return feature_set.filter(f'{column_name} is not null and {column_name} != \'\'')
            else:
                return feature_set.filter(f'{column_name} is not null')
        elif f'{column_name} == null' == action_code:
            if is_string:
                return feature_set.filter(f'{column_name} is null or {column_name} == \'\'')
            else:
                return feature_set.filter(f'{column_name} is null')
        elif action_code.startswith(f'{column_name} contains'):
            value = action_code.split(' ')[2].strip('"')
            return feature_set.filter(f'{column_name} like \'%{value}%\'')
    return feature_set.filter(action_code)


def transform_first(feature_set, transformer_action, sort_options={}):
    return transform_agg(feature_set, transformer_action, F.first)


def transform_group(feature_set, transformer_action, sort_options={}):
    new_fields = __create_new_struct_fields(feature_set, transformer_action)
    schema = StructType(feature_set.schema.fields + new_fields)
    groupby_columns = transformer_action['action_options']['groupby_columns']

    def execute_transform(df):
        from mage_ai.data_cleaner.transformer_actions.base import BaseAction

        if sort_options.get('columns') is not None:
            sort_columns = sort_options.get('columns')
            sort_ascending = sort_options.get('ascending', True)
            df = df.sort_values(sort_columns, ascending=sort_ascending)

        df = BaseAction(transformer_action).execute(df)

        for field in new_fields:
            if field.name not in df:
                df[field.name] = 0
        return df

    return feature_set.groupby(groupby_columns).applyInPandas(execute_transform, schema=schema)


def transform_last(feature_set, transformer_action, sort_options={}):
    return transform_agg(feature_set, transformer_action, F.last)


def transform_max(feature_set, transformer_action, sort_options={}):
    return transform_agg(feature_set, transformer_action, F.max)


def transform_median(feature_set, transformer_action, sort_options={}):
    def __cal_median(col):
        return F.expr(f'percentile_approx({col}, 0.5)')
    return transform_agg(feature_set, transformer_action, __cal_median, use_string_col_name=True)


def transform_min(feature_set, transformer_action, sort_options={}):
    return transform_agg(feature_set, transformer_action, F.min)


def transform_noop(feature_set, transformer_action, sort_options={}):
    return feature_set


def transform_remove(feature_set, transformer_action, sort_options={}):
    return feature_set.drop(*transformer_action['action_arguments'])


def transform_select(feature_set, transformer_action, sort_options={}):
    return feature_set.select(*transformer_action['action_arguments'])


def transform_sort(feature_set, transformer_action, sort_options={}):
    ascending = transformer_action['action_options'].get('ascending', True)
    return feature_set.sort(transformer_action['action_arguments'], ascending=ascending)


def transform_sum(feature_set, transformer_action, sort_options={}):
    return transform_agg(feature_set, transformer_action, F.sum)


def transform_with_partitions(feature_set, transformer_action, sort_options={}):
    window = Window.partitionBy(ROW_NUMBER_LIT_COLUMN).orderBy(ROW_NUMBER_LIT_COLUMN)

    fs = feature_set.withColumn(
        ROW_NUMBER_LIT_COLUMN,
        F.lit('a'),
    ).withColumn(
        ROW_NUMBER_COLUMN,
        F.row_number().over(window),
    ).withColumn(
        GROUP_MOD_COLUMN,
        expr(f'mod({ROW_NUMBER_COLUMN}, 1000)'),
    )

    new_fields = __create_new_struct_fields(feature_set, transformer_action)
    schema = StructType(fs.schema.fields + new_fields)

    def execute_transform(df):
        from mage_ai.data_cleaner.transformer_actions.base import BaseAction
        df = BaseAction(transformer_action).execute(df)

        return df

    df_filtered = fs.groupby(GROUP_MOD_COLUMN).applyInPandas(execute_transform, schema=schema)

    return df_filtered.drop(GROUP_MOD_COLUMN, ROW_NUMBER_COLUMN, ROW_NUMBER_LIT_COLUMN)


TRANSFORMER_FUNCTION_MAPPING = {
    'add': {
        # 'distance_between': transform_add_distance_between,
    },
    'average': transform_average,
    'clean_column_name': transform_clean_column_name,
    'count': transform_count,
    'count_distinct': transform_count_distinct,
    'drop_duplicate': transform_drop_duplicate,
    'expand_column': transform_group,
    # 'filter': transform_filter,
    'first': transform_first,
    # 'group': transform_group,
    'last': transform_last,
    'max': transform_max,
    'median': transform_median,
    'min': transform_min,
    'remove': transform_remove,
    'remove_outliers': transform_noop,
    'select': transform_select,
    'shift_down': transform_group,
    'sort': transform_sort,
    'sum': transform_sum,
}


def transform(feature_set, transformer_action, sort_options={}):
    action_type = transformer_action['action_type']
    if action_type == 'add':
        udf = transformer_action['action_options'].get('udf', None)
        if udf and udf in TRANSFORMER_FUNCTION_MAPPING['add']:
            return TRANSFORMER_FUNCTION_MAPPING['add'][udf](
                feature_set,
                transformer_action,
                sort_options=sort_options,
            )
    elif action_type in TRANSFORMER_FUNCTION_MAPPING:
        return TRANSFORMER_FUNCTION_MAPPING[action_type](
            feature_set,
            transformer_action,
            sort_options=sort_options,
        )
    return transform_with_partitions(feature_set, transformer_action)



from typing import Dict
import os
import yaml

DEFAULT_INSTANCE_TYPE = 'r5.4xlarge'
MAX_CLUSTERS_ON_SPOT_INSTANCES = 40


class EmrConfig:
    def __init__(self, config_path: str = None, config: Dict = None):
        if config is None:
            self.config_path = config_path
            if self.config_path is None:
                raise Exception(
                    'Please provide a config_path or a config dictionary to initialize'
                    ' an EmrConfig object',
                )
            if not os.path.exists(self.config_path):
                raise Exception(f'EMR config {self.config_path} does not exist.')
            with open(self.config_path) as fp:
                config = yaml.full_load(fp) or {}

        if 'emr_config' in config:
            config = config['emr_config']
        self.config = config
        self.master_security_group = config.get('master_security_group')
        if self.master_security_group is None:
            raise Exception(
                'Please provide master_security_group in config file to initialize an EmrConfig',
            )
        self.slave_security_group = \
            config.get('slave_security_group') or self.master_security_group
        self.master_instance_type = config.get('master_instance_type', DEFAULT_INSTANCE_TYPE)
        self.slave_instance_type = config.get('slave_instance_type', DEFAULT_INSTANCE_TYPE)

    def get_instances_config(self, cluster_count, idle_timeout=0):
        market = 'SPOT' if cluster_count < MAX_CLUSTERS_ON_SPOT_INSTANCES else 'ON_DEMAND'
        instances_config = {
            'KeepJobFlowAliveWhenNoSteps': False,
            'EmrManagedMasterSecurityGroup': self.master_security_group,
            'EmrManagedSlaveSecurityGroup': self.slave_security_group,
            'InstanceGroups': [
                dict(
                    Name='AmazonEMRMaster',
                    Market=market,
                    InstanceRole='MASTER',
                    InstanceType=self.master_instance_type,
                    InstanceCount=1,
                    Configurations=[
                        {
                            'Classification': 'spark-defaults',
                            'Properties': {
                                'spark.driver.memory': '32000M',
                                'spark.driver.maxResultSize': '0',
                                'spark.executor.memory': '32000M',
                            },
                        },
                    ],
                ),
                dict(
                    Name='AmazonEMRCore',
                    Market=market,
                    InstanceRole='CORE',
                    InstanceType=self.slave_instance_type,
                    InstanceCount=1,
                    Configurations=[
                        {
                            'Classification': 'spark-defaults',
                            'Properties': {
                                'spark.driver.memory': '32000M',
                                'spark.driver.maxResultSize': '0',
                                'spark.executor.memory': '32000M',
                            },
                        },
                    ],
                ),
            ],
        }
        return instances_config
